#!/usr/bin/python3

import argparse
import roypy
import time
import queue
from sample_camera_info import print_camera_info
from roypy_sample_utils import CameraOpener, add_camera_opener_options, select_use_case
from roypy_platform_utils import PlatformHelper

import numpy as np
import matplotlib.pyplot as plt

import imageio.v3 as iio
import open3d as o3d
from skimage import color
import matplotlib.pyplot as plt

# Clase escoltant encarregada de controlar els events de visualització
class MyListener(roypy.IDepthDataListener):
    # Constructor que agafa les dades de la imatge de profunditat internament
    def __init__(self, q):
        super(MyListener, self).__init__()
        self.queue = q
        self.figSetup = False

    def onNewData(self, data):
        pc = data.npoints ()
        p = pc[:,:,2]
        self.queue.put(p)

    def paint (self, data):
        # Funció per la visualització de les dades
        if not self.figSetup:
            self.fig = plt.figure(1)
            self.im = plt.imshow(data)

            plt.show(block = False)
            plt.draw()
            self.figSetup = True
        else:
            self.im.set_data(data)
            self.fig.canvas.draw()
        # Temps necessari per fer la pintura correctament
        plt.pause(0.001)

def main ():
    # Establim els paràmetres necessaris de la càmera per la captura i imprimim la informació
    platformhelper = PlatformHelper()
    parser = argparse.ArgumentParser (usage = __doc__)
    add_camera_opener_options (parser)
    parser.add_argument ("--seconds", type=int, default=15, help="duration to capture data")
    options = parser.parse_args()
    print(options)
    opener = CameraOpener (options)
    cam = opener.open_camera ()

    print_camera_info (cam)
    print("isConnected", cam.isConnected())
    print("getFrameRate", cam.getFrameRate())

    curUseCase = select_use_case(cam)

    try:
        replay = cam.asReplay()
        print ("Using a recording")
        print ("Framecount : ", replay.frameCount())
        print ("File version : ", replay.getFileVersion())
    except SystemError:
        print ("Using a live camera")
    
    # Usem aquesta cua per sincronitzar el codi amb el fil principal
    q = queue.Queue()
    
    # Cridem a la funció escoltant
    l = MyListener(q)
    cam.registerDataListener(l)

    print ("Setting use case : " + curUseCase)
    cam.setUseCase(curUseCase)

    # Comença la captura de la càmera
    cam.startCapture()
    # La càmera de profunditat romandrà oberta 15/3 segons
    process_event_queue (q, l, options.seconds/3)
    # Finalitza la captura de la càmera
    cam.stopCapture()
    
def lod_mesh_export(mesh, lods, extension, path):
    '''Aquesta funció serveix per generar nivells de detall.
    La función prendrà com a paràmetres una malla, una llista de LoD 
    (nombre objectiu de triangles), el format dels arxius resultants 
    i la ruta on escriure els arxius'''
    
    mesh_lods={}
    for i in lods:
        
        # Aquesta funció dóna la possibilitat de guardar els resultats
        # en una variable en cas que python necessiti més processament 
        mesh_lod = mesh.simplify_quadric_decimation(i)
        
        # Exportem les dades a la ubicació especificada 
        o3d.io.write_triangle_mesh(path+"lod_"+str(i)+extension, mesh_lod)
        
        mesh_lods[i]=mesh_lod
        
    print("generation of "+str(i)+" LoD successful")
    return mesh_lods

    
def mesh(pcd):
    
    # Calculem el radi necessari en funció de les distàncies entre els putns
    distances = pcd.compute_nearest_neighbor_distance()
    avg_dist = np.mean(distances)
    radius = 1.01 * avg_dist
    
    # Creem una malla i la guardem en la variable bpa_mesh
    bpa_mesh = o3d.geometry.TriangleMesh.create_from_point_cloud_ball_pivoting(pcd,o3d.utility.DoubleVector([radius, radius * 2]))
    
    # Abans d'exportar la malla, podem reduïr el resultat a 100mil triangles 
    dec_mesh = bpa_mesh.simplify_quadric_decimation(100000)
    
    # Programem la consistència de la malla
    dec_mesh.remove_degenerate_triangles()
    dec_mesh.remove_duplicated_triangles()
    dec_mesh.remove_duplicated_vertices()
    dec_mesh.remove_non_manifold_edges()
    
    # Afegim color
    dec_mesh.paint_uniform_color([1, 1, 1])
    
    # Exportem les dades indicant el nombre del fitxer que volguem crear,
    # La extensió que volguem i la malla a exportar. 
    output_path="C:/Users/Asus/Desktop/"
    o3d.io.write_triangle_mesh(output_path+"bpa_mesh.ply", dec_mesh)
    
    # Podem escollir diferents números de triangles per a cada malla 
    my_lods = lod_mesh_export(bpa_mesh, [100000,50000,10000,1000,100], ".ply", output_path)
    my_lods2 = lod_mesh_export(bpa_mesh, [8000,800,300], ".ply", output_path)
    my_lods3 = lod_mesh_export(bpa_mesh, [24000,2400,1200], ".ply", output_path)
    my_lods4 = lod_mesh_export(bpa_mesh, [3672000,367200,183600], ".ply", output_path)
    my_lods5 = lod_mesh_export(bpa_mesh, [3672000*100000000000000,367200*100000000000000,183600*100000000000000], ".ply", output_path)
    
    # Visualitzem la malla amb cen triangles
    o3d.visualization.draw_geometries([my_lods5[183600*100000000000000]])

def point_claud(depth_image):
    # Paràmetres de profunditat de la càmera:
    FX_DEPTH = 217.9000701904297
    FY_DEPTH = 217.9000701904297
    CX_DEPTH = 105.79347229003906
    CY_DEPTH = 87.20672607421875
    
    # Vector de coordenades dels punts del núvol:
    pcd = []
    height = depth_image.shape[0] 
    width = depth_image.shape[1]
    threshold = 2
    for i in range(height):
        for j in range(width):
            z = depth_image[i][j]
            x = (j - CX_DEPTH) * z / FX_DEPTH
            y = (i - CY_DEPTH) * z / FY_DEPTH
            pcd.append([x, y, z])
                 
    # Creació del núvol de punts inicialment buit
    pcd_o3d = o3d.geometry.PointCloud()
    
    # Assignem els punts de la imatge al nostre núvol
    pcd_o3d.points = o3d.utility.Vector3dVector(pcd) 

    # Visualitzem gràcies a la llibreria open 3d:
    o3d.visualization.draw_geometries([pcd_o3d])
    
    # A continuació seleccionem solament l'objecte del núvol de punts general
    points = np.asarray(pcd_o3d.points)

    # Definim l'esfera amb centre i radi
    center = np.array([0.008, 0.026, 0.436])
    radius = 0.15

    # Calculem les distàncies al centre, establint els nous punts
    distances = np.linalg.norm(points - center, axis=1)
    pcd_o3d.points = o3d.utility.Vector3dVector(points[distances <= radius])

    # Visualitzem de nou
    o3d.visualization.draw_geometries([pcd_o3d])
    
    # Establim les normals del núvol de punts
    pcd_o3d.estimate_normals(search_param=o3d.geometry.KDTreeSearchParamHybrid(radius=0.1, max_nn=30))
    
    # Fem el downsampling vóxel uniforme
    uniform_pcd = pcd_o3d.uniform_down_sample(every_k_points=2)
    
    # Visualitzem ara amb el downsampling 
    o3d.visualization.draw_geometries([uniform_pcd])
    
    # Transformem el núvol de punts en la malla
    mesh(uniform_pcd)

    
def process_event_queue (q, painter, seconds):
    # Creem un bucle durant el temps indicat
    t_end = time.time() + seconds
    while time.time() < t_end:
        try:
            # Bloquegem fins recuperar els elements de la cua
            # Com a forma de sincronització
            if len(q.queue) == 0:
                item = q.get(True, 1)
            else:
                for i in range (0, len (q.queue)):
                    item = q.get(True, 1)
        except queue.Empty:
            break
        else:
            # Mostrem la pantalla de captura
            painter.paint (item)
    
    # Transformem la imatge de profunditat en el núvol de punts
    point_claud(item)

if (__name__ == "__main__"):
    main()
